@@ -240,7 +240,7 @@ BOOL cmd_stackframe(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
         char* key = hash_loop_key(it);
 
         char*obj_kind[16] = {
-            NULL, "var", "array", "hash", "list", "native function", "block", "file dicriptor", "job", "object", "memchecker", "function", "class", NULL, NULL, NULL
+            NULL, "var", "array", "hash", "list", "native function", "block", "file dicriptor", "job", "object", "memchecker", "function", "class", "external program", "completion", "self matching completion"
         };
         snprintf(buf, BUFSIZ, "%s: %s\n", key, obj_kind[TYPE(object2)]);
 
@@ -412,7 +412,9 @@ BOOL cmd_eval(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
         int rcode = 0;
         sObject* block = BLOCK_NEW_STACK();
         int sline = 1;
-        if(parse(buf, "eval", &sline, block, NULL)) {
+        char buf2[BUFSIZ];
+        snprintf(buf2, BUFSIZ, "%s %d: eval", runinfo->mSName, runinfo->mSLine);
+        if(parse(buf, buf2, &sline, block, NULL)) {
             if(!run(block, nextin, nextout, &rcode, runinfo->mCurrentObject, runinfo->mRunningObject)) {
                 runinfo->mRCode = rcode;
                 stack_end_stack();
@@ -420,7 +422,7 @@ BOOL cmd_eval(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
             }
         }
         else {
-            err_msg("parser error\n", runinfo->mSName, runinfo->mSLine);
+            err_msg_adding("parser error\n", runinfo->mSName, runinfo->mSLine);
             stack_end_stack();
             return FALSE;
         }
@@ -435,7 +437,9 @@ BOOL cmd_eval(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
         int rcode = 0;
         sObject* block = BLOCK_NEW_STACK();
         int sline = 1;
-        if(parse(buf, "eval", &sline, block, NULL)) {
+        char buf2[BUFSIZ];
+        snprintf(buf2, BUFSIZ, "%s %d: eval", runinfo->mSName, runinfo->mSLine);
+        if(parse(buf, buf2, &sline, block, NULL)) {
             if(!run(block, nextin, nextout, &rcode, runinfo->mCurrentObject, runinfo->mRunningObject)) {
                 runinfo->mRCode = rcode;
                 stack_end_stack();
@@ -443,7 +447,7 @@ BOOL cmd_eval(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
             }
         }
         else {
-            err_msg("parser error\n", runinfo->mSName, runinfo->mSLine);
+            err_msg_adding("parser error\n", runinfo->mSName, runinfo->mSLine);
             stack_end_stack();
             return FALSE;
         }
