@@ -34,7 +34,7 @@ BOOL cmd_mshow(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
 
         char buf[BUFSIZ];
         char*obj_kind[16] = {
-            NULL, "var", "array", "hash", "list", "native function", "block", "file dicriptor", "job", "object", "memchecker", "function", "class", "external program", NULL, 
+            NULL, "var", "array", "hash", "list", "native function", "block", "file dicriptor", "job", "object", "memchecker", "function", "class", "external program", "completion", "self matching completion"
         };
         snprintf(buf, BUFSIZ, "%s: %s\n", key, obj_kind[TYPE(object2)]);
 
@@ -320,6 +320,7 @@ BOOL cmd_inherit(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
 
         case T_NFUN:
         case T_BLOCK:
+        case T_COMPLETION:
             err_msg("There is not a parent object", runinfo->mSName, runinfo->mSLine);
             return FALSE;
 
@@ -683,7 +684,7 @@ BOOL cmd_ref(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
                 if(type >= 0) {
                     char buf[128];
                     char* obj_kind[16] = {
-                        NULL, "var", "array", "hash", "list", "native function", "block", "file dicriptor", "job", "object", "memchecker", "function", "class", "external program", NULL
+                        NULL, "var", "array", "hash", "list", "native function", "block", "file dicriptor", "job", "object", "memchecker", "function", "class", "external program", "completion", "self matching completion"
                     };
                     snprintf(buf, 128, "%s\n", obj_kind[type]);
 
