@@ -213,18 +213,87 @@ BOOL cmd_length(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
         code = kEucjp;
     }
 
+    enum eLineField lf = gLineField;
+    if(sCommand_option_item(command, "-Lw")) {
+        lf = kCRLF;
+    }
+    else if(sCommand_option_item(command, "-Lm")) {
+        lf = kCR;
+    }
+    else if(sCommand_option_item(command, "-Lu")) {
+        lf = kLF;
+    }
+    else if(sCommand_option_item(command, "-La")) {
+        lf = kBel;
+    }
+
     /// output
     if(runinfo->mFilter) {
-        char* arg = SFD(nextin).mBuf;
-        int len = str_kanjilen(code, arg);
+        /// line number
+        if(sCommand_option_item(command, "-line-num")) {
+            int result = 0;
+            char* p = SFD(nextin).mBuf;
+            if(lf == kCRLF) {
+                while(1) {
+                    if(*p == '\r' && *(p+1) == '\n') {
+                        p+=2;
+                        result++;
+                    }
+                    else if(*p == 0) {
+                        break;
+                    }
+                    else {
+                        p++;
+                    }
+                }
+            }
+            else {
+                char split_char;
+                if(lf == kLF) {
+                    split_char = '\n';
+                }
+                else if(lf == kCR) {
+                    split_char = '\r';
+                }
+                else {
+                    split_char = '\a';
+                }
 
-        char buf[128];
-        snprintf(buf, 128, "%d\n", len);
+                while(1) {
+                    if(*p == split_char) {
+                        p++;
+                        result++;
+                    }
+                    else if(*p == 0) {
+                        break;
+                    }
+                    else {
+                        p++;
+                    }
+                }
+            }
 
-        if(!fd_write(nextout, buf)) {
-            err_msg("signal interrupt", runinfo->mSName, runinfo->mSLine);
-            runinfo->mRCode = RCODE_SIGNAL_INTERRUPT;
-            return FALSE;
+            char buf[128];
+            snprintf(buf, 128, "%d\n", result);
+
+            if(!fd_write(nextout, buf)) {
+                err_msg("signal interrupt", runinfo->mSName, runinfo->mSLine);
+                runinfo->mRCode = RCODE_SIGNAL_INTERRUPT;
+                return FALSE;
+            }
+        }
+        else {
+            char* arg = SFD(nextin).mBuf;
+            int len = str_kanjilen(code, arg);
+
+            char buf[128];
+            snprintf(buf, 128, "%d\n", len);
+
+            if(!fd_write(nextout, buf)) {
+                err_msg("signal interrupt", runinfo->mSName, runinfo->mSLine);
+                runinfo->mRCode = RCODE_SIGNAL_INTERRUPT;
+                return FALSE;
+            }
         }
 
         runinfo->mRCode = 0;
@@ -1020,9 +1089,6 @@ BOOL sub_oneline(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
                                 onig_free(reg);
                                 return FALSE;
                             }
-                        }
-
-                        if(!quiet) {
                             if(!fd_write(nextout, string_c_str(sub_str))) {
                                 err_msg("signal interrupt", runinfo->mSName, runinfo->mSLine);
                                 runinfo->mRCode = RCODE_SIGNAL_INTERRUPT;
@@ -1061,14 +1127,12 @@ BOOL sub_oneline(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
 
                         onig_region_free(region, 1);
 
-                        if(!global) {
-                            if(!quiet) {
-                                if(!fd_write(nextout, p)) {
-                                    err_msg("signal interrupt", runinfo->mSName, runinfo->mSLine);
-                                    runinfo->mRCode = RCODE_SIGNAL_INTERRUPT;
-                                    onig_free(reg);
-                                    return FALSE;
-                                }
+                        if(!global && !quiet) {
+                            if(!fd_write(nextout, p)) {
+                                err_msg("signal interrupt", runinfo->mSName, runinfo->mSLine);
+                                runinfo->mRCode = RCODE_SIGNAL_INTERRUPT;
+                                onig_free(reg);
+                                return FALSE;
                             }
                             break;
                         }
@@ -1233,9 +1297,6 @@ BOOL sub_multiline(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
                             onig_free(reg);
                             return FALSE;
                         }
-                    }
-
-                    if(!quiet) {
                         if(!fd_write(nextout, string_c_str(sub_str))) {
                             err_msg("signal interrupt", runinfo->mSName, runinfo->mSLine);
                             runinfo->mRCode = RCODE_SIGNAL_INTERRUPT;
@@ -1274,14 +1335,12 @@ BOOL sub_multiline(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
 
                     onig_region_free(region, 1);
 
-                    if(!global) {
-                        if(!quiet) {
-                            if(!fd_write(nextout, p)) {
-                                err_msg("signal interrupt", runinfo->mSName, runinfo->mSLine);
-                                runinfo->mRCode = RCODE_SIGNAL_INTERRUPT;
-                                onig_free(reg);
-                                return FALSE;
-                            }
+                    if(!global && !quiet) {
+                        if(!fd_write(nextout, p)) {
+                            err_msg("signal interrupt", runinfo->mSName, runinfo->mSLine);
+                            runinfo->mRCode = RCODE_SIGNAL_INTERRUPT;
+                            onig_free(reg);
+                            return FALSE;
                         }
                         break;
                     }
@@ -1581,178 +1640,6 @@ BOOL cmd_scan(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
     }
 }
 
-BOOL split_oneline(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
-{
-    sCommand* command = runinfo->mCommand;
-
-    char* field = "\n";
-    enum eLineField lf = gLineField;
-    if(sCommand_option_item(command, "-Lw")) {
-        lf = kCRLF;
-        field = "\r\n";
-    }
-    else if(sCommand_option_item(command, "-Lm")) {
-        lf = kCR;
-        field = "\r";
-    }
-    else if(sCommand_option_item(command, "-Lu")) {
-        lf = kLF;
-        field = "\n";
-    }
-    else if(sCommand_option_item(command, "-La")) {
-        lf = kBel;
-        field = "\a";
-    }
-
-    enum eKanjiCode code = gKanjiCode;
-    if(sCommand_option_item(command, "-byte")) {
-        code = kByte;
-    }
-    else if(sCommand_option_item(command, "-utf8")) {
-        code = kUtf8;
-    }
-    else if(sCommand_option_item(command, "-sjis")) {
-        code = kSjis;
-    }
-    else if(sCommand_option_item(command, "-eucjp")) {
-        code = kEucjp;
-    }
-
-    if(runinfo->mFilter) {
-        char* regex;
-
-        if(command->mArgsNumRuntime == 1) {
-            regex = "\\s+";
-        }
-        else if(command->mArgsNumRuntime >= 2) {
-            regex = command->mArgsRuntime[1];
-        }
-
-        regex_t* reg;
-        int r = get_onig_regex(&reg, command, regex);
-
-        if(r == ONIG_NORMAL) {
-            fd_split(nextin, kLF);
-
-            int i;
-            for(i=0; i<vector_size(SFD(nextin).mLines); i++) {
-                char* target;
-                char* p = target = vector_item(SFD(nextin).mLines, i);
-
-                while(*p) {
-                    OnigRegion* region = onig_region_new();
-
-                    int r2 = onig_search(reg, target
-                       , target + strlen(target)
-                       , p, p + strlen(p)
-                       , region, ONIG_OPTION_NONE);
-                       
-                    if(r2 == ONIG_MISMATCH) {
-                        onig_region_free(region, 1);
-
-                        if(!fd_write(nextout, p)) {
-                            err_msg("singal interrupt", runinfo->mSName, runinfo->mSLine);
-                            runinfo->mRCode = RCODE_SIGNAL_INTERRUPT;
-                            onig_free(reg);
-                            return FALSE;
-                        }
-
-                        if(!fd_write(nextout, field)) {
-                            err_msg("singal interrupt", runinfo->mSName, runinfo->mSLine);
-                            runinfo->mRCode = RCODE_SIGNAL_INTERRUPT;
-                            onig_free(reg);
-                            return FALSE;
-                        }
-                        break;
-                    }
-                    else {
-                        if(region->beg[0] == region->end[0]) {
-                            char* end_byte = str_kanjipos2pointer(code, target + region->beg[0], 1);
-                            int size = end_byte - p;
-
-                            if(size > 0) {
-                                if(!fd_write3(nextout, p, size)) {
-                                    err_msg("singal interrupt", runinfo->mSName, runinfo->mSLine);
-                                    runinfo->mRCode = RCODE_SIGNAL_INTERRUPT;
-                                    onig_region_free(region, 1);
-                                    onig_free(reg);
-                                    return FALSE;
-                                }
-
-                                if(!fd_write(nextout, field)) {
-                                    err_msg("singal interrupt", runinfo->mSName, runinfo->mSLine);
-                                    runinfo->mRCode = RCODE_SIGNAL_INTERRUPT;
-                                    onig_region_free(region, 1);
-                                    onig_free(reg);
-                                    return FALSE;
-                                }
-                            }
-
-                            p = target + region->end[0] + size;
-                        }
-                        else {
-                            int size = region->beg[0] - (p-target);
-
-                            if(size > 0) {
-                                if(!fd_write3(nextout, p, size)) {
-                                    err_msg("singal interrupt", runinfo->mSName, runinfo->mSLine);
-                                    runinfo->mRCode = RCODE_SIGNAL_INTERRUPT;
-                                    onig_region_free(region, 1);
-                                    onig_free(reg);
-                                    return FALSE;
-                                }
-
-                                if(!fd_write(nextout, field)) {
-                                    err_msg("singal interrupt", runinfo->mSName, runinfo->mSLine);
-                                    runinfo->mRCode = RCODE_SIGNAL_INTERRUPT;
-                                    onig_region_free(region, 1);
-                                    onig_free(reg);
-                                    return FALSE;
-                                }
-                            }
-
-                            p = target + region->end[0];
-                        }
-
-                        if(region->num_regs > 1) {
-                            /// group strings ///
-                            int i;
-                            for(i=1; i<region->num_regs; i++) {
-                                const int size = region->end[i]-region->beg[i];
-                                if(size > 0) {
-                                    if(!fd_write3(nextout, target + region->beg[i], size)) {
-                                        err_msg("singal interrupt", runinfo->mSName, runinfo->mSLine);
-                                        runinfo->mRCode = RCODE_SIGNAL_INTERRUPT;
-                                        onig_region_free(region, 1);
-                                        onig_free(reg);
-                                        return FALSE;
-                                    }
-
-                                    if(!fd_write(nextout, field)) {
-                                        err_msg("singal interrupt", runinfo->mSName, runinfo->mSLine);
-                                        runinfo->mRCode = RCODE_SIGNAL_INTERRUPT;
-                                        onig_region_free(region, 1);
-                                        onig_free(reg);
-                                        return FALSE;
-                                    }
-                                }
-                            }
-                        }
-                    }
-
-                    onig_region_free(region, 1);
-                }
-            }
-
-            runinfo->mRCode = 0;
-        }
-
-        onig_free(reg);
-    }
-
-    return TRUE;
-}
-
 BOOL split_multiline(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
 {
     sCommand* command = runinfo->mCommand;
@@ -1807,7 +1694,7 @@ BOOL split_multiline(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
             char* target;
             char* p = target = SFD(nextin).mBuf;
 
-            while(*p) {
+            while(1) {
                 OnigRegion* region = onig_region_new();
 
                 int r2 = onig_search(reg, target
@@ -1818,11 +1705,13 @@ BOOL split_multiline(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
                 if(r2 == ONIG_MISMATCH) {
                     onig_region_free(region, 1);
 
-                    if(!fd_write(nextout, p)) {
-                        err_msg("singal interrupt", runinfo->mSName, runinfo->mSLine);
-                        runinfo->mRCode = RCODE_SIGNAL_INTERRUPT;
-                        onig_free(reg);
-                        return FALSE;
+                    if(*p) {
+                        if(!fd_write(nextout, p)) {
+                            err_msg("singal interrupt", runinfo->mSName, runinfo->mSLine);
+                            runinfo->mRCode = RCODE_SIGNAL_INTERRUPT;
+                            onig_free(reg);
+                            return FALSE;
+                        }
                     }
 
                     if(!fd_write(nextout, field)) {
@@ -1856,7 +1745,8 @@ BOOL split_multiline(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
                             }
                         }
 
-                        p = target + region->end[0] + size;
+                        int size2 = end_byte - (target + region->beg[0]);
+                        p = target + region->end[0] + size2;
                     }
                     else {
                         int size = region->beg[0] - (p-target);
@@ -1878,6 +1768,15 @@ BOOL split_multiline(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
                                 return FALSE;
                             }
                         }
+                        else if(size == 0) {
+                            if(!fd_write(nextout, field)) {
+                                err_msg("singal interrupt", runinfo->mSName, runinfo->mSLine);
+                                runinfo->mRCode = RCODE_SIGNAL_INTERRUPT;
+                                onig_region_free(region, 1);
+                                onig_free(reg);
+                                return FALSE;
+                            }
+                        }
 
                         p = target + region->end[0];
                     }
@@ -1924,12 +1823,7 @@ BOOL cmd_split(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
 {
     sCommand* command = runinfo->mCommand;
 
-//    if(sCommand_option_item(command, "-multi-line")) {
-        return split_multiline(nextin, nextout, runinfo);
-//    }
-//    else {
-//        return split_oneline(nextin, nextout, runinfo);
-//    }
+    return split_multiline(nextin, nextout, runinfo);
 }
 
 BOOL cmd_add(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
@@ -2249,3 +2143,151 @@ BOOL cmd_rows(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
 
     return TRUE;
 }
+
+#if defined(HAVE_MIGEMO_H)
+#include <migemo.h>
+
+migemo* gMigemo;
+static regex_t* gReg;
+
+void migemo_init()
+{
+    char buf[PATH_MAX];
+    char migemodir[PATH_MAX];
+    gMigemo = migemo_open(NULL);
+
+    snprintf(migemodir, PATH_MAX, "%s", SYSTEM_MIGEMODIR);
+
+    snprintf(buf, PATH_MAX, "%s/utf-8/migemo-dict", migemodir);
+    if(migemo_load(gMigemo, MIGEMO_DICTID_MIGEMO, buf) == MIGEMO_DICTID_INVALID) {
+        fprintf(stderr, "%s is not found\n", buf);
+        exit(1);
+    }
+    snprintf(buf, PATH_MAX, "%s/utf-8/roma2hira.dat", migemodir);
+    if(migemo_load(gMigemo, MIGEMO_DICTID_ROMA2HIRA, buf) == MIGEMO_DICTID_INVALID) {
+        fprintf(stderr, "%s is not found\n", buf);
+        exit(1);
+    }
+    snprintf(buf, PATH_MAX, "%s/utf-8/hira2kata.dat", migemodir);
+    if(migemo_load(gMigemo, MIGEMO_DICTID_HIRA2KATA, buf) == MIGEMO_DICTID_INVALID) {
+        fprintf(stderr, "%s is not found\n", buf);
+        exit(1);
+    }
+    snprintf(buf, PATH_MAX, "%s/utf-8/han2zen.dat", migemodir);
+    if(migemo_load(gMigemo, MIGEMO_DICTID_HAN2ZEN, buf) == MIGEMO_DICTID_INVALID) {
+        fprintf(stderr, "%s is not found\n", buf);
+        exit(1);
+    }
+}
+
+void migemo_final()
+{
+    onig_end();
+    migemo_close(gMigemo);
+    if(gReg) onig_free(gReg);
+    gReg = NULL;
+}
+
+BOOL cmd_migemo_match(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
+{
+    sCommand* command = runinfo->mCommand;
+
+    BOOL quiet = sCommand_option_item(command, "-quiet");
+
+    if(runinfo->mFilter && command->mArgsNumRuntime == 2) {
+        runinfo->mRCode = RCODE_NFUN_FALSE;
+        char* target = SFD(nextin).mBuf;
+        char* regex = command->mArgsRuntime[1];
+
+        if(regex[0] == 0) {
+            runinfo->mRCode = 0;
+            if(!quiet) {
+                char buf[1024];
+                snprintf(buf, 1024, "%d\n%d\n", 0, strlen(target));
+                if(!fd_write(nextout, buf)) {
+                    err_msg("signal interrupt", runinfo->mSName, runinfo->mSLine);
+                    runinfo->mRCode = RCODE_SIGNAL_INTERRUPT;
+                    return FALSE;
+                }
+            }
+        }
+        else {
+            OnigUChar * p = migemo_query(gMigemo, regex);
+            if(p == NULL) {
+                err_msg("migemo query failed", runinfo->mSName, runinfo->mSLine);
+                migemo_release(gMigemo, (unsigned char*) p);
+                return FALSE;
+            }
+
+            /// modify query ///
+            char* p2 = MALLOC(strlen(p)*2 + 1);
+
+            char* _p = p;
+            char* _p2 = p2;
+
+            while(*_p) {
+                if(*_p == '+') {
+                    *_p2++ = '\\';
+                    *_p2++ = *_p++;
+                }
+                else {
+                    *_p2++ = *_p++;
+                }
+            }
+            *_p2 = 0;
+
+            /// make regex ///
+            regex_t* reg;
+            OnigErrorInfo err_info;
+
+            int r = onig_new(&reg, p2, p2 + strlen(p2), ONIG_OPTION_DEFAULT, ONIG_ENCODING_UTF8, ONIG_SYNTAX_DEFAULT,  &err_info);
+
+            FREE(p2);
+            migemo_release(gMigemo, (unsigned char*) p);
+
+            if(r != ONIG_NORMAL && r != 0) {
+                err_msg("regex of migemo query failed", runinfo->mSName, runinfo->mSLine);
+                onig_free(reg);
+                return FALSE;
+            }
+
+            OnigRegion* region = onig_region_new();
+            int r2 = onig_search(reg, target, target + strlen(target), target, target + strlen(target), region, ONIG_OPTION_NONE);
+
+            if(r2 == 0) {
+                runinfo->mRCode = 0;
+                if(!quiet) {
+                    char buf[1024];
+                    snprintf(buf, 1024, "%d\n%d\n", region->beg[0], region->end[0]);
+                    if(!fd_write(nextout, buf)) {
+                        err_msg("signal interrupt", runinfo->mSName, runinfo->mSLine);
+                        runinfo->mRCode = RCODE_SIGNAL_INTERRUPT;
+                        onig_region_free(region, 1);
+                        onig_free(reg);
+                        return FALSE;
+                    }
+                }
+            }
+            else {
+                if(!quiet) {
+                    char buf[1024];
+                    snprintf(buf, 1024, "-1\n-1\n");
+                    if(!fd_write(nextout, buf)) {
+                        err_msg("signal interrupt", runinfo->mSName, runinfo->mSLine);
+                        runinfo->mRCode = RCODE_SIGNAL_INTERRUPT;
+                        onig_region_free(region, 1);
+                        onig_free(reg);
+                        return FALSE;
+                    }
+                }
+            }
+
+            onig_region_free(region, 1);
+            onig_free(reg);
+        }
+    }
+
+    return TRUE;
+}
+
+#endif
