@@ -18,8 +18,10 @@ static void sig_int_readline(int signo)
     rl_redisplay();
 }
 
-static int readline_signal()
+int readline_signal()
 {
+    pipes_set_signal();
+
     struct sigaction sa2;
     memset(&sa2, 0, sizeof(sa2));
     sa2.sa_handler = sig_int_readline;
@@ -84,7 +86,7 @@ static char* prompt()
             fprintf(stderr, "%s", string_c_str(gErrMsg));
         }
     }
-    pipes_restore_signal_default();
+    readline_signal();
 
     mreset_tty();
     readline_signal();
@@ -96,9 +98,9 @@ static char* prompt()
     return buf;
 }
 
-void pipes_readline_interface(char** argv, int argc)
+void pipes_readline_interface(char** argv, int argc, BOOL no_runtime_script)
 {
-    pipes_init(kATConsoleApp);
+    pipes_init(kATConsoleApp, no_runtime_script);
 
     /// start interactive shell ///
     mreset_tty();
@@ -190,6 +192,7 @@ void pipes_readline_interface(char** argv, int argc)
                 hash_put(SFUN(fun).mLocalObjects, "ARGV", argv2);
 
                 if(!run(block, gStdin, gStdout, &rcode, gCurrentObject, fun)) {
+                    readline_signal();
                     if(rcode == RCODE_BREAK) {
                         fprintf(stderr, "invalid break. Not in a loop\n");
                     }
@@ -206,7 +209,7 @@ void pipes_readline_interface(char** argv, int argc)
                         fprintf(stderr, "%s", string_c_str(gErrMsg));
                     }
                 }
-                pipes_restore_signal_default();
+                readline_signal();
 
                 (void)vector_pop_back(gStackFrames);
             }
@@ -255,9 +258,9 @@ void pipes_readline_interface(char** argv, int argc)
     pipes_final();
 }
 
-void pipes_load_file(char* fname, char** argv, int argc)
+void pipes_load_file(char* fname, char** argv, int argc, BOOL no_runtime_script)
 {
-    pipes_init(kATOptC);
+    pipes_init(kATOptC, no_runtime_script);
 
     pipes_set_signal_optc();
     sRunInfo runinfo;
@@ -291,9 +294,9 @@ void pipes_load_file(char* fname, char** argv, int argc)
     pipes_final();
 }
 
-void pipes_opt_c(char* cmd, char** argv, int argc)
+void pipes_opt_c(char* cmd, char** argv, int argc, BOOL no_runtime_script)
 {
-    pipes_init(kATOptC);
+    pipes_init(kATOptC, no_runtime_script);
 
     stack_start_stack();
     int rcode = 0;
