@@ -2,42 +2,49 @@
 #include "pipes/pipes.h"
 #include <stdio.h>
 
-void version()
+static void version()
 {
-    printf("pipes version %s\n", getenv("PIPES_VERSION"));
+    printf("Pipes version %s\n", getenv("PIPES_VERSION"));
 }
 
 int main(int argc, char** argv)
 {
     CHECKML_BEGIN(FALSE);
 
-    setenv("PIPES_VERSION", "1.0.1", 1);
+    setenv("PIPES_VERSION", "1.0.3", 1);
     setenv("PIPES_DATAROOTDIR", DATAROOTDIR, 1);
 
     srandom((unsigned)time(NULL));
 
     char* optc = NULL;
+    BOOL no_runtime_script = FALSE;
+    char* file = NULL;
     int i;
-    for(i=0; i<argc; i++) {
+    for(i=1; i<argc; i++) {
         if(strcmp(argv[i], "-c") == 0 && i+1 < argc) {
             optc = argv[i+1];
-            break;
         }
         else if(strcmp(argv[i], "--version") == 0) {
             version();
             exit(0);
         }
+        else if(strcmp(argv[i], "-nr") == 0) {
+            no_runtime_script = TRUE;
+        }
+        else {
+            file = argv[i];
+        }
     }
 
     if(optc) {
-        pipes_opt_c(optc, argv, argc);
+        pipes_opt_c(optc, argv, argc, no_runtime_script);
     }
-    else if(argc >= 2) {
-        pipes_load_file(argv[1], argv, argc);
+    else if(file) {
+        pipes_load_file(file, argv, argc, no_runtime_script);
     }
     else {
         setenv("PIPES_RUNNING_AS_INTRACTIVE_MODE", "1", 1);
-        pipes_readline_interface(argv, argc);
+        pipes_readline_interface(argv, argc, no_runtime_script);
     }
 
     CHECKML_END();
