@@ -143,6 +143,9 @@ void uobject_root_init(sObject* self)
     hash_put(self, "selector", NFUN_NEW_GC(cmd_selector, NULL, TRUE));
     hash_put(self, "sort", NFUN_NEW_GC(cmd_sort, NULL, TRUE));
     hash_put(self, "readline", NFUN_NEW_GC(cmd_readline, NULL, TRUE));
+#if defined(HAVE_MIGEMO_H)
+    hash_put(self, "migemo_match", NFUN_NEW_GC(cmd_migemo_match, NULL, TRUE));
+#endif
 
     hash_put(self, "sub", NFUN_NEW_GC(cmd_sub, NULL, TRUE));
     hash_put(self, "time", NFUN_NEW_GC(cmd_time, NULL, TRUE));
