@@ -23,7 +23,7 @@ static void pipes_read_rc_core(char* path)
 
     if(path && access(path, R_OK) == 0) {
         memset(&runinfo, 0, sizeof(sRunInfo));
-        runinfo.mSName = "pipes.pi(system)";
+        runinfo.mSName = path;
         runinfo.mCurrentObject = gRootObject;
 
         if(!load_file(path, gStdin, gStdout, &runinfo, argv, argc))
@@ -82,7 +82,7 @@ static void pipes_read_rc()
     pipes_restore_signal_default();
 }
 
-void pipes_init(enum eAppType app_type)
+void pipes_init(enum eAppType app_type, BOOL no_runtime_script)
 {
     stack_init(1);;
     stack_start_stack();
@@ -114,11 +114,21 @@ void pipes_init(enum eAppType app_type)
         exit(1);
     }
 
-    pipes_read_rc();
+#if defined(HAVE_MIGEMO_H)
+    migemo_init();
+#endif
+
+    if(!no_runtime_script) {
+        pipes_read_rc();
+    }
 }
 
 void pipes_final()
 {
+#if defined(HAVE_MIGEMO_H)
+    migemo_final();
+#endif
+
     mcurses_final();
     run_final();
     gc_final();
