@@ -317,6 +317,7 @@ typedef struct {
 #define T_FUN 11
 #define T_CLASS 12
 #define T_EXTERNAL 13
+#define T_COMPLETION 14
 
 typedef struct _sObject {
     int mFlg;                         // contains a kind of the above and a mark flag
@@ -419,6 +420,9 @@ extern char * gCommandKindStrs[kCommandMax];
 
 BOOL pipes_rehash(char* sname, int sline);
 
+#if defined(HAVE_MIGEMO_H)
+BOOL cmd_migemo_match(sObject* nextin, sObject* nextout, sRunInfo* runinfo);
+#endif
 BOOL cmd_readline(sObject* nextin, sObject* nextout, sRunInfo* runinfo);
 BOOL cmd_mshow(sObject* nextin, sObject* nextout, sRunInfo* runinfo);
 BOOL cmd_mrun(sObject* nextin, sObject* nextout, sRunInfo* runinfo);
@@ -540,7 +544,7 @@ enum eAppType { kATOptC, kATXApp, kATConsoleApp };
 
 extern enum eAppType gAppType;
 
-void pipes_init(enum eAppType app_type);
+void pipes_init(enum eAppType app_type, BOOL no_runtime_script);
 void pipes_final();
 
 void run_init(enum eAppType app_type);
@@ -608,6 +612,7 @@ BOOL parse(char* p, char* sname, int* sline, sObject* block, sObject** current_o
 void pipes_kill_all_jobs();
 
 extern sObject* gRootObject;
+extern sObject* gCompletionObject;
 extern sObject* gPipesObject;
 extern sObject* gCurrentObject;
 extern sObject* gObject;      // Object class object
@@ -616,9 +621,9 @@ extern sObject* gStackFrames;
 BOOL bufsiz_write(int fd, char* buf);
 
 BOOL load_file(char* fname, sObject* nextin, sObject* nextout, sRunInfo* runinfo, char** argv, int argc);
-void pipes_load_file(char* fname, char** argv, int argc);
-void pipes_opt_c(char* cmd, char** argv, int argc);
-void pipes_readline_interface(char** argv, int argc);
+void pipes_load_file(char* fname, char** argv, int argc, BOOL no_runtime_script);
+void pipes_opt_c(char* cmd, char** argv, int argc, BOOL no_runtime_script);
+void pipes_readline_interface(char** argv, int argc, BOOL no_runtime_script);
 
 void fd_split(sObject* self, enum eLineField lf);
 
@@ -690,4 +695,11 @@ extern sObject* gPrompt;
 
 extern sObject* gGlobalPipe;
 
+#if defined(HAVE_MIGEMO_H)
+void migemo_init();
+void migemo_final();
+#endif
+
+int readline_signal();
+
 #endif
