@@ -38,7 +38,6 @@ static BOOL name_sort(void* left, void* right)
 }
 
 static sObject* gUserCompletionNextout;
-static sObject* gUserCompBlocks;
 static sObject* gReadlineCurrentObject;
 
 static char* message_completion(const char* text, int stat)
@@ -70,7 +69,7 @@ static char* message_completion(const char* text, int stat)
             int i;
             for(i=1; i<command->mMessagesNum; i++) {
                 object = hash_item(object, command->mMessages[i]);
-                if(object == NULL) break;
+                if(object == NULL || TYPE(object) != T_UOBJECT) break;
                 string_push_back(messages, command->mMessages[i]);
                 string_push_back(messages, "->");
             }
@@ -200,7 +199,12 @@ static char* user_completion(const char* text, int stat)
         index++;
 
         if(!strncmp(text, candidate, wordlen)) {
-            rl_completion_append_character = ' ';
+            if(candidate[strlen(candidate)-1] == '/') {
+                rl_completion_append_character = 0;
+            }
+            else {
+                rl_completion_append_character = ' ';
+            }
             return strdup(candidate);
         }
     }
@@ -405,12 +409,29 @@ char** readline_on_complete(const char* text, int start, int end)
 
             const int command_num = command->mArgsNum + (SBLOCK(gReadlineBlock).mCompletionFlags & COMPLETION_FLAGS_FILE_COMPLETION ? 1:0);
 
-            sObject* ucomp_block;
+            sObject* ucompletion;
             if(command->mArgsNum > 0) {
-                ucomp_block = hash_item(gUserCompBlocks, command->mArgs[0]);
+                sObject* object = gCompletionObject;
+                int i;
+                for(i=0; i<command->mMessagesNum; i++) {
+                    object = hash_item(object, command->mMessages[i]);
+                    if(object == NULL || TYPE(object) != T_UOBJECT) {
+                        break;
+                    }
+                }
+                if(object && TYPE(object) == T_UOBJECT) {
+                    ucompletion = hash_item(object, command->mArgs[0]);
+
+                    if(ucompletion == NULL) {
+                        ucompletion = hash_item(object, "*");
+                    }
+                }
+                else {
+                    ucompletion = NULL;
+                }
             }
             else {
-                ucomp_block = NULL;
+                ucompletion = NULL;
             }
 
             if(SBLOCK(gReadlineBlock).mCompletionFlags & (COMPLETION_FLAGS_ENV)) {
@@ -418,7 +439,19 @@ char** readline_on_complete(const char* text, int start, int end)
                 stack_end_stack();
                 return result;
             }
-            else if(ucomp_block) {
+            else if(command_num < 2 || SBLOCK(gReadlineBlock).mCompletionFlags & (COMPLETION_FLAGS_NEXT_PIPE|COMPLETION_FLAGS_NEXT_STATMENT)) {
+                if(command->mMessagesNum > 0 && !(SBLOCK(gReadlineBlock).mCompletionFlags & (COMPLETION_FLAGS_NEXT_PIPE|COMPLETION_FLAGS_NEXT_STATMENT))) {
+                    char** result = rl_completion_matches(text, message_completion);
+                    stack_end_stack();
+                    return result;
+                }
+                else {
+                    char** result = rl_completion_matches(text, program_completion);
+                    stack_end_stack();
+                    return result;
+                }
+            }
+            else if(ucompletion && TYPE(ucompletion) == T_COMPLETION) {
                 sObject* nextin = FD_NEW_STACK();
                 if(!fd_write(nextin, string_c_str(cmdline))) {
                     stack_end_stack();
@@ -432,29 +465,35 @@ char** readline_on_complete(const char* text, int start, int end)
                 //uobject_init(stackframe);
                 SFUN(fun).mLocalObjects = stackframe;
 
+                sObject* argv = VECTOR_NEW_GC(16, FALSE);
+                if(command->mArgsNum > 0) {
+                    vector_add(argv, STRING_NEW_GC(command->mArgs[0], FALSE));
+                    vector_add(argv, STRING_NEW_GC(command->mArgs[command->mArgsNum-1], FALSE));
+                }
+                else {
+                    vector_add(argv, STRING_NEW_GC("", FALSE));
+                    vector_add(argv, STRING_NEW_GC("", FALSE));
+                }
+                hash_put(SFUN(fun).mLocalObjects, "ARGV", argv);
+
                 int rcode = 0;
-                if(!run(ucomp_block, nextin, nextout, &rcode, gReadlineCurrentObject, fun)) {
+                pipes_set_signal();
+                if(!run(ucompletion, nextin, nextout, &rcode, gReadlineCurrentObject, fun)) {
+                    readline_signal();
+                    fprintf(stderr, "\nrun time error\n");
+                    fprintf(stderr, "%s", string_c_str(gErrMsg));
                     vector_pop_back(gStackFrames);
                     stack_end_stack();
                     return NULL;
                 }
+                readline_signal();
                 vector_pop_back(gStackFrames);
 
                 fd_split(nextout, kLF);
 
-                gUserCompletionNextout = nextout;
-                char** result = rl_completion_matches(text, user_completion);
-                stack_end_stack();
-                return result;
-            }
-            else if(command_num < 2 || SBLOCK(gReadlineBlock).mCompletionFlags & (COMPLETION_FLAGS_NEXT_PIPE|COMPLETION_FLAGS_NEXT_STATMENT)) {
-                if(command->mMessagesNum > 0 && !(SBLOCK(gReadlineBlock).mCompletionFlags & (COMPLETION_FLAGS_NEXT_PIPE|COMPLETION_FLAGS_NEXT_STATMENT))) {
-                    char** result = rl_completion_matches(text, message_completion);
-                    stack_end_stack();
-                    return result;
-                }
-                else {
-                    char** result = rl_completion_matches(text, program_completion);
+                if(TYPE(ucompletion) == T_COMPLETION) {
+                    gUserCompletionNextout = nextout;
+                    char** result = rl_completion_matches(text, user_completion);
                     stack_end_stack();
                     return result;
                 }
@@ -516,9 +555,46 @@ BOOL cmd_completion(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
 
         int i;
         for(i=1; i<command->mArgsNumRuntime; i++) {
-            char* cmd = command->mArgsRuntime[i];
+            sObject* object = gCompletionObject;
 
-            hash_put(gUserCompBlocks, cmd, block_clone_gc(block, T_BLOCK, FALSE));
+            sObject* str = STRING_NEW_GC("", FALSE);
+            char* p = command->mArgsRuntime[i];
+            while(*p) {
+                if(*p == '-' && *(p+1) == '>') {
+                    if(string_c_str(str)[0] == 0) {
+                        err_msg("invalid object name", runinfo->mSName, runinfo->mSLine);
+                        return FALSE;
+                    }
+                    p+=2;
+
+                    sObject* object2 = hash_item(object, string_c_str(str));
+                    if(object2 == NULL || TYPE(object2) !=T_UOBJECT) {
+                        sObject* new_object = UOBJECT_NEW_GC(8, object, string_c_str(str), TRUE);
+                        hash_put(object, string_c_str(str), new_object);
+                        uobject_init(new_object);
+
+                        object = new_object;
+                    }
+                    else {
+                        object = object2;
+                    }
+                    string_put(str, "");
+                }
+                else {
+                    string_push_back2(str, *p);
+                    p++;
+                }
+            }
+            if(object && TYPE(object) == T_UOBJECT && string_c_str(str)[0] != 0) {
+                hash_put(object, string_c_str(str), block_clone_gc(block, T_COMPLETION, FALSE));
+            }
+            else {
+                char buf[BUFSIZ];
+                snprintf(buf, BUFSIZ, "There is no %s object", command->mArgsRuntime[i]);
+                err_msg(buf, runinfo->mSName, runinfo->mSLine);
+
+                return FALSE;
+            }
         }
 
         runinfo->mRCode = 0;
@@ -563,7 +639,8 @@ static int readline_macro(int count, int key)
                 fprintf(stderr, "%s", string_c_str(gErrMsg));
             }
         }
-        pipes_restore_signal_default();
+        readline_signal();
+        //pipes_restore_signal_default();
         (void)vector_pop_back(gStackFrames);
     }
     else {
@@ -591,9 +668,6 @@ void pipes_readline_init(BOOL runtime_script)
     rl_completion_append_character= ' ';
     rl_char_is_quoted_p = (rl_linebuf_func_t*)lftp_char_is_quoted;
 
-    gUserCompBlocks = HASH_NEW_GC(16, FALSE);
-    hash_put(gPipesObject, "_ucomp_blocks", gUserCompBlocks);
-
     rl_bind_key('x'-'a'+1, readline_macro);
 }
 
