@@ -62,7 +62,7 @@ sObject* fd_new_from_stack()
 
     *SFD(self).mBuf = 0;
 
-    SFD(self).mLines = VECTOR_NEW_STACK(32);
+    SFD(self).mLines = VECTOR_NEW_MALLOC(32);
 
     return self;
 }
@@ -74,6 +74,8 @@ void fd_delete_stack(sObject* self)
     for(i=0; i<vector_size(v); i++) {
         FREE(vector_item(v, i));
     }
+    vector_delete_malloc(SFD(self).mLines);
+
     FREE(SFD(self).mBuf);
 }
 
@@ -344,7 +346,7 @@ static BOOL nextout_reader(sObject* nextout, int* pipeoutfds, sRunInfo* runinfo)
         char buf[BUFSIZ+1];
         while(1) {
             if(gPipesSigInt) {
-                err_msg("signal interrupt", runinfo->mSName, runinfo->mSLine);
+                err_msg("signal interrupt1", runinfo->mSName, runinfo->mSLine);
                 gPipesSigInt = FALSE;
                 runinfo->mRCode = RCODE_SIGNAL_INTERRUPT;
                 return FALSE;
@@ -353,7 +355,7 @@ static BOOL nextout_reader(sObject* nextout, int* pipeoutfds, sRunInfo* runinfo)
             if(r < 0) {
                 if(errno == EINTR) {
                     if(!fd_write(nextout, buf)) {
-                        err_msg("signal interrupt", runinfo->mSName, runinfo->mSLine);
+                        err_msg("signal interrupt2", runinfo->mSName, runinfo->mSLine);
                         runinfo->mRCode = RCODE_SIGNAL_INTERRUPT;
                         return FALSE;
                     }
@@ -367,7 +369,7 @@ static BOOL nextout_reader(sObject* nextout, int* pipeoutfds, sRunInfo* runinfo)
             buf[r] = 0;
 
             if(!fd_write(nextout, buf)) {
-                err_msg("signal interrupt", runinfo->mSName, runinfo->mSLine);
+                err_msg("signal interrupt3", runinfo->mSName, runinfo->mSLine);
                 runinfo->mRCode = RCODE_SIGNAL_INTERRUPT;
                 return FALSE;
             }
@@ -440,7 +442,7 @@ static BOOL nextin_writer(pid_t pid, sObject* nextin, int* pipeinfds, int* pipeo
                 kill(pid2, SIGCONT);
             }
             else if(WIFSIGNALED(status)) {
-                err_msg("signal interrupt", runinfo->mSName, runinfo->mSLine);
+                err_msg("signal interrupt4", runinfo->mSName, runinfo->mSLine);
                 return FALSE;
             }
             else {
@@ -454,7 +456,7 @@ static BOOL nextin_writer(pid_t pid, sObject* nextin, int* pipeinfds, int* pipeo
 
         if(WIFSIGNALED(status) || WIFSTOPPED(status))
         {
-            err_msg("signal interrupt", runinfo->mSName, runinfo->mSLine);
+            err_msg("signal interrupt5", runinfo->mSName, runinfo->mSLine);
             return FALSE;
         }
     }
@@ -476,7 +478,7 @@ static BOOL wait_child_program(pid_t pid, int nextout2, sRunInfo* runinfo)
                     kill(pid, SIGCONT);
                 }
                 else if(WIFSIGNALED(status)) {
-                    err_msg("signal interrupt", runinfo->mSName, runinfo->mSLine);
+                    err_msg("signal interrupt6", runinfo->mSName, runinfo->mSLine);
                     return FALSE;
                 }
                 else {
@@ -489,7 +491,7 @@ static BOOL wait_child_program(pid_t pid, int nextout2, sRunInfo* runinfo)
             pid = waitpid(pid, &status, WUNTRACED);
 
             if(WIFSTOPPED(status)) {
-                err_msg("signal interrupt", runinfo->mSName, runinfo->mSLine);
+                err_msg("signal interrupt7", runinfo->mSName, runinfo->mSLine);
 
                 kill(pid, SIGKILL);
                 pid = waitpid(pid, &status, WUNTRACED);
@@ -500,7 +502,7 @@ static BOOL wait_child_program(pid_t pid, int nextout2, sRunInfo* runinfo)
                 return FALSE;
             }
             else if(WIFSIGNALED(status)) {  // a pipes external program which is not a last command can't be stopped by CTRL-Z
-                err_msg("signal interrupt", runinfo->mSName, runinfo->mSLine);
+                err_msg("signal interrupt8", runinfo->mSName, runinfo->mSLine);
 
                 if(tcsetpgrp(0, getpid()) < 0) {
                     perror("tcsetpgrp(pipes)");
@@ -889,12 +891,12 @@ static BOOL run_object(sObject* object, sObject* reciever, sObject* current_obje
 
         case T_STRING: {
             if(!fd_write(nextout, string_c_str(object))) {
-                err_msg("signal interrupt", runinfo->mSName, runinfo->mSLine);
+                err_msg("signal interrupt9", runinfo->mSName, runinfo->mSLine);
                 runinfo->mRCode = RCODE_SIGNAL_INTERRUPT;
                 return FALSE;
             }
             if(!fd_write(nextout, "\n")) {
-                err_msg("signal interrupt", runinfo->mSName, runinfo->mSLine);
+                err_msg("signal interrupt10", runinfo->mSName, runinfo->mSLine);
                 runinfo->mRCode = RCODE_SIGNAL_INTERRUPT;
                 return FALSE;
             }
@@ -974,7 +976,7 @@ static BOOL statment_tree(sStatment* statment, sObject* pipein, sObject* pipeout
     if(statment->mFlags & STATMENT_CONTEXTPIPE) {
         if(pipein == gStdin && SFD(gStdin).mBufLen == 0) {
             if(!stdin_read_out()) {
-                err_msg("signal interrupt", runinfo->mSName, runinfo->mSLine);
+                err_msg("signal interrupt11", runinfo->mSName, runinfo->mSLine);
                 runinfo->mRCode = RCODE_SIGNAL_INTERRUPT;
                 return FALSE;
             }
@@ -985,7 +987,7 @@ static BOOL statment_tree(sStatment* statment, sObject* pipein, sObject* pipeout
     else if(statment->mFlags & STATMENT_GLOBALPIPEIN) {
         nextin = FD_NEW_STACK();
         if(!fd_write(nextin, SFD(gGlobalPipe).mBuf)) {
-            err_msg("signal interrupt", runinfo->mSName, runinfo->mSLine);
+            err_msg("signal interrupt12", runinfo->mSName, runinfo->mSLine);
             runinfo->mRCode = RCODE_SIGNAL_INTERRUPT;
             return FALSE;
         }
@@ -1143,7 +1145,7 @@ BOOL run(sObject* block, sObject* pipein, sObject* pipeout, int* rcode, sObject*
         /// flash stdout and stderr ///
         if(pipeout == gStdout) {
             if(!fd_flash(gStdout, 1)) {
-                err_msg("signal interrupt", runinfo.mSName, runinfo.mSLine);
+                err_msg("signal interrupt13", runinfo.mSName, runinfo.mSLine);
                 *rcode = RCODE_SIGNAL_INTERRUPT;
                 stack_end_stack();
                 nest_level--;
@@ -1153,7 +1155,7 @@ BOOL run(sObject* block, sObject* pipein, sObject* pipeout, int* rcode, sObject*
 
         if(nest_level == 1) {
             if(!fd_flash(gStderr, 2)) {
-                err_msg("signal interrupt", runinfo.mSName, runinfo.mSLine);
+                err_msg("signal interrupt14", runinfo.mSName, runinfo.mSLine);
                 *rcode = RCODE_SIGNAL_INTERRUPT;
                 stack_end_stack();
                 nest_level--;
@@ -1171,7 +1173,7 @@ BOOL run(sObject* block, sObject* pipein, sObject* pipeout, int* rcode, sObject*
         }
         else if(runinfo.mRCode == 128+SIGINT || gPipesSigInt) {
             gPipesSigInt = FALSE;
-            err_msg("signal interrupt", runinfo.mSName, runinfo.mSLine);
+            err_msg("signal interrupt15", runinfo.mSName, runinfo.mSLine);
             *rcode = RCODE_SIGNAL_INTERRUPT;
             stack_end_stack();
             nest_level--;
@@ -1374,7 +1376,7 @@ BOOL load_file(char* fname, sObject* nextin, sObject* nextout, sRunInfo* runinfo
     char* buf;
     int buf_size;
     if(!bufsiz_read(fd, ALLOC &buf, &buf_size)) {
-        err_msg("signal interrupt", runinfo->mSName, runinfo->mSLine);
+        err_msg("signal interrupt16", runinfo->mSName, runinfo->mSLine);
         runinfo->mRCode = RCODE_SIGNAL_INTERRUPT;
         FREE(buf);
         close(fd);
